/* 
 * Process Variable Library with JVisu Socket Server
 * 
 * Copyright (C) 2003 Thomas Gleixner <tglx@linutronix.de>
 *           (C) 2005 Robert Schwebel <r.schwebel@pengutronix.de>
 *		
 * PURPOSE
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <pv_types.h>


struct pv_bs_priv {
	int fd;
	int mmap;
};


static int pv_bs_exit(pv_backing_store_t *store) {
	struct pv_bs_priv *priv = store->priv;

	if(priv->mmap)
		munmap(store->map, store->size);

	close(priv->fd);

	return 0;
}


static int pv_bs_read(pv_backing_store_t *store, off_t offset, size_t len) {
	struct pv_bs_priv *priv = store->priv;
	int ret;

	VFPRINTF(2, stderr, "%s: off: %d len: %d\n",__FUNCTION__,(int)offset,len);

	ret = lseek(priv->fd, offset, SEEK_SET);
	if(ret < 0) {
		perror("lseek");
		return ret;
	}

	ret = read(priv->fd, store->map + offset, len);
	if(ret < 0) {
		perror("write");
		return ret;
	}

	return 0;
}


static int pv_bs_write(pv_backing_store_t *store, off_t offset, size_t len) {
	struct pv_bs_priv *priv = store->priv;
	int ret;

	VFPRINTF(2, stderr, "%s: off: %d len: %d\n",__FUNCTION__,(int)offset,len);

	ret = lseek(priv->fd, offset, SEEK_SET);
	if(ret < 0) {
		perror("lseek");
		return ret;
	}

	ret = write(priv->fd,store->map + offset, len);
	if(ret < 0) {
		perror("write");
		return ret;
	}

	return 0;
}


int pv_bs_init(pv_backing_store_t *store, const char *filename) {
	struct pv_bs_priv *priv;
	int ret = 0;

	printf("initialising backing store\n");

	priv = malloc(sizeof(*priv));
	if(!priv) {
		perror("malloc");
		ret = -1;
		goto exit;
	}

	memset(priv, 0, sizeof(priv));
	store->priv = priv;
	store->exit = pv_bs_exit;
	store->read = pv_bs_read;
	store->write = pv_bs_write;

	ret = open(filename, O_RDWR | O_SYNC);
	if(ret < 0) {
		perror("open");
		goto exit_free;
	}
	priv->fd = ret;

	store->map = mmap(NULL, store->size, PROT_READ | PROT_WRITE, 
		         MAP_SHARED, priv->fd, 0);

	if(store->map == MAP_FAILED) {
		printf("could not mmap, need shared memory segment\n");
		store->map = NULL;
	} else
		priv->mmap = 1;

	return 0;

 exit_free:
	free(priv);
 exit:
	return ret;
}
